@@ -37,7 +37,7 @@ from dpys import utils
 RED = 0xD40C00
 BLUE = 0x0000FF
 GREEN = 0x32C12C
-version = "5.4.1"
+version = "5.4.5"
 EPHEMERAL = True
 warnings_db: aiosqlite.Connection
 muted_db: aiosqlite.Connection
@@ -112,19 +112,18 @@ class admin:
             return
         if len(str(reason)) > 256:
             reason = reason[:256]
+        if not isinstance(inter.guild.get_role(role_add), discord.Role):
+            return
+        await member.add_roles(inter.guild.get_role(role_add))
+        if role_remove is not None:
+            with contextlib.suppress(discord.Forbidden, discord.HTTPException):
+                await member.remove_roles(inter.guild.get_role(role_remove))
+        if reason is None:
+            await inter.response.send_message(msg or f"Muted {str(member)}.", ephemeral=EPHEMERAL)
         else:
-            if not isinstance(inter.guild.get_role(role_add), discord.Role):
-                return
-            await member.add_roles(inter.guild.get_role(role_add))
-            if role_remove is not None:
-                with contextlib.suppress(discord.Forbidden, discord.HTTPException):
-                    await member.remove_roles(inter.guild.get_role(role_remove))
-            if reason is None:
-                await inter.response.send_message(msg or f"Muted {str(member)}.", ephemeral=EPHEMERAL)
-            else:
-                await inter.response.send_message(
-                    msg or f"Muted {member.name}#{member.discriminator}. Reason: {reason}",
-                    ephemeral=EPHEMERAL)
+            await inter.response.send_message(
+                msg or f"Muted {member.name}#{member.discriminator}. Reason: {reason}",
+                ephemeral=EPHEMERAL)
 
     @staticmethod
     async def unmute(inter: ApplicationCommandInteraction, member: discord.Member, role_remove: int,
@@ -181,6 +180,32 @@ class admin:
             message = msg or f"Banned {member.name}#{member.discriminator}. Reason: {reason}"
         await inter.response.send_message(message, ephemeral=EPHEMERAL)
 
+    @staticmethod
+    async def timeout(inter: ApplicationCommandInteraction, member: discord.Member,
+                      duration: Union[float, datetime.timedelta] = None, until: datetime.datetime = None,
+                      reason: typing.Optional[str] = None, msg: str = None) -> None:
+        if len(str(reason)) > 256:
+            reason = reason[:256]
+        if duration is not None:
+            await member.timeout(duration=duration, reason=reason)
+            if isinstance(duration, datetime.timedelta):
+                end_timeout = utils.get_discord_date((datetime.datetime.now() + duration).timestamp())
+            else:
+                end_timeout = utils.get_discord_date(
+                    (datetime.datetime.now() + datetime.timedelta(seconds=duration)).timestamp())
+        elif until is not None:
+            await member.timeout(until=until, reason=reason)
+            end_timeout = utils.get_discord_date(until.timestamp())
+        else:
+            await member.timeout(reason=reason)
+            await inter.response.send_message(f"Removed timeout from {str(member)}.", ephemeral=EPHEMERAL)
+            return
+        if reason is None:
+            message = msg or f"Timed out {str(member)} until {end_timeout}."
+        else:
+            message = msg or f"Timed out {str(member)} until {end_timeout}. Reason: {reason}"
+        await inter.response.send_message(message, ephemeral=EPHEMERAL)
+
     @staticmethod
     async def softban(inter: ApplicationCommandInteraction, member: discord.Member,
                       reason: typing.Optional[str] = None, msg: str = None) -> None:
@@ -230,7 +255,7 @@ class curse:
         words = word.replace(" ", "")
         words = words.split(",")
         words = set(words)
-        curses = await utils.GuildData.curse_set(inter.guild.id)
+        curses = await utils.GuildData.curse_set(inter.guild.id, db)
         for x in words:
             if x in curses:
                 msg = f"{x} is already in the list."
@@ -414,9 +439,9 @@ class mute_on_join:
 class warnings:
     class Punishment:
         def __init__(self, punishment: str, duration: typing.Optional[int] = None):
-            if punishment.startswith("temp") and duration is None:
+            if (punishment.startswith("temp") or punishment == "timeout") and duration is None:
                 raise Exception("duration cannot be None for temporary punishments.")
-            if punishment not in ["temp_ban", "temp_mute", "mute", "ban", "kick"]:
+            if punishment not in ["temp_ban", "temp_mute", "mute", "ban", "kick", "timeout"]:
                 raise Exception("Invalid punishment.")
             self.punishment = punishment
             if not punishment.startswith("temp"):
@@ -555,7 +580,8 @@ class warnings:
     async def punish(inter: ApplicationCommandInteraction, member: discord.Member,
                      punishments: typing.Mapping[int, Punishment],
                      add_role: typing.Optional[int] = None, remove_role: typing.Optional[int] = None,
-                     before: Optional[Callable[[int, Punishment, discord.Member], Awaitable[None]]] = None) -> None:
+                     before: Optional[Callable[
+                         [int, Punishment, discord.Member], Awaitable[Optional[discord.Message]]]] = None) -> None:
         memberid = str(member.id)
         guild = str(inter.guild.id)
         db = warnings_db
@@ -582,15 +608,20 @@ class warnings:
             return
         if punishments[warnings_number].duration is not None:
             if punishments[warnings_number].punishment == "temp_ban":
+                msg = await before(warnings_number, punishments[warnings_number], member)
                 time = punishments[warnings_number].duration
-                await member.ban(reason=f"You have received {warnings_number} warnings.")
+                try:
+                    await member.ban(reason=f"You have received {warnings_number} warning(s).")
+                except (discord.Forbidden, discord.HTTPException) as e:
+                    with contextlib.suppress(Exception):
+                        await msg.delete()
+                    raise e
                 time = datetime.datetime.now() + datetime.timedelta(seconds=time)
                 await db.execute("INSERT INTO tempban (guild,member,time) VALUES (?,?,?)",
                                  (guild, memberid, time))
                 await db.commit()
-                await before(warnings_number, punishments[warnings_number], member)
                 return
-            else:
+            elif punishments[warnings_number].punishment == "temp_mute":
                 add_role = inter.guild.get_role(add_role)
                 if not isinstance(add_role, discord.Role):
                     return
@@ -610,14 +641,29 @@ class warnings:
                                      (guild, memberid, time))
                     await db.commit()
                     await before(warnings_number, punishments[warnings_number], member)
-        else:
-            if punishments[warnings_number].punishment == "ban":
-                await member.ban(reason=f"You have received {warnings_number} warnings.")
+            else:
+                time = punishments[warnings_number].duration
+                await member.timeout(reason=f"You have received {warnings_number} warning(s).", duration=time)
                 await before(warnings_number, punishments[warnings_number], member)
                 return
+        else:
+            if punishments[warnings_number].punishment == "ban":
+                msg = await before(warnings_number, punishments[warnings_number], member)
+                try:
+                    await member.ban(reason=f"You have received {warnings_number} warning(s).")
+                except (discord.Forbidden, discord.HTTPException) as e:
+                    with contextlib.suppress(Exception):
+                        await msg.delete()
+                    raise e
+                return
             if punishments[warnings_number].punishment == "kick":
-                await member.kick(reason=f"You have received {warnings_number} warnings.")
-                await before(warnings_number, punishments[warnings_number], member)
+                msg = await before(warnings_number, punishments[warnings_number], member)
+                try:
+                    await member.kick(reason=f"You have received {warnings_number} warning(s).")
+                except (discord.Forbidden, discord.HTTPException) as e:
+                    with contextlib.suppress(Exception):
+                        await msg.delete()
+                    raise e
                 return
             if punishments[warnings_number].punishment == "mute":
                 add_role = inter.guild.get_role(add_role)
